#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Debug interface for twisted applications
#
# Copyright (c) 2006 JAS
#
# Author: Petr Vokac <vokac@kmlinux.fjfi.cvut.cz>
#
# $Id$
#
import sys
import logging
from twisted.internet import reactor, protocol, interfaces
from twisted.enterprise import adbapi
from twisted.protocols.basic import LineReceiver


class Redirect:
    def __init__(self):
        self.buf = ''
    def write(self, buf):
        self.buf += buf
    def getBuffer(self):
        buf = self.buf
        self.buf = ''
        return buf

class DebugProtocol(LineReceiver):

    COMMANDS = [ "quit" ]

    def __init__(self):
        self.cmd = ''
        self.stdoutOrig = sys.stdout
        self.stdoutRedir = Redirect()

    def __printPrefix(self, prefix):
        delimiter = self.delimiter
        self.delimiter = ''
        self.sendLine(prefix)
        self.delimiter = delimiter

    def connectionMade(self):
        # self.sendLine("Write python expression or call predefined command.")
        # self.sendLine("Debugs: %s\n", ", ".join(DebugProtocol.COMMANDS))
        self.__printPrefix('>>> ')

    def connectionLost(self, reason):
        pass

    def lineReceived(self, line):
        logging.getLogger().debug(line)
        app = self.factory.app
        if line.lower() == 'quit':
            self.sendLine('bye')
            self.transport.loseConnection()
            return
        try:
            prefix = '>>> '
            buf = ''
            if self.cmd == '':
                if line != '':
                    sys.stdout = self.stdoutRedir
                    eval(compile(line, '<prompt>', 'single'), globals(), locals())
                    sys.stdout = self.stdoutOrig
                    buf = self.stdoutRedir.getBuffer()
            else:
                if line != '':
                    prefix = '... '
                    self.cmd = "%s\n%s" % (self.cmd, line)
                else:
                    sys.stdout = self.stdoutRedir
                    eval(compile(self.cmd, '<prompt>', 'exec'), globals(), locals())
                    sys.stdout = self.stdoutOrig
                    self.cmd = ''
                    buf = self.stdoutRedir.getBuffer()
            if buf != '':
                if buf[len(buf)-1] == "\n": buf = buf[:-1]
                logging.getLogger().debug(buf)
                for line in buf.split("\n"):
                    self.sendLine(line)
            self.__printPrefix(prefix)
        except EOFError:
            self.sendLine('bye')
            self.transport.loseConnection()
        except SyntaxError, e:
            if self.cmd == '':
                self.__printPrefix('... ')
                self.cmd = line
            else:
                for line in str(e).split("\n"):
                    self.sendLine(line)
                self.__printPrefix('>>> ')
                self.cmd = ''
        except Exception, e:
            self.sendLine(str(e))
            self.__printPrefix('>>> ')
        self.stdoutRedir.getBuffer()
        sys.stdout = self.stdoutOrig


class DebugFactory(protocol.Factory):

    protocol = DebugProtocol

    def __init__(self, app):
        self.app = app

    def startFactory(self):
        pass

    def stopFactory(self):
        pass
